@@ -52,8 +52,8 @@ S_dump_trie(pTHX_ const struct _reg_trie_data *trie, HV *widecharmap,
             AV *revcharmap, U32 depth)
 {
     U32 state;
-    SV *sv=sv_newmortal();
-    int colwidth= widecharmap ? 6 : 4;
+    SV *sv = sv_newmortal();
+    int colwidth = widecharmap ? 6 : 4;
     U16 word;
     DECLARE_AND_GET_RE_DEBUG_FLAGS;
 
@@ -128,7 +128,7 @@ S_dump_trie(pTHX_ const struct _reg_trie_data *trie, HV *widecharmap,
     }
     Perl_re_indentf( aTHX_  "word_info N:(prev,len)=",
                                 depth);
-    for (word=1; word <= trie->wordcount; word++) {
+    for (word = 1; word <= trie->wordcount; word++) {
         Perl_re_printf( aTHX_  " %d:(%d,%d)",
             (int)word, (int)(trie->wordinfo[word].prev),
             (int)(trie->wordinfo[word].len));
@@ -147,8 +147,8 @@ S_dump_trie_interim_list(pTHX_ const struct _reg_trie_data *trie,
                          U32 depth)
 {
     U32 state;
-    SV *sv=sv_newmortal();
-    int colwidth= widecharmap ? 6 : 4;
+    SV *sv = sv_newmortal();
+    int colwidth = widecharmap ? 6 : 4;
     DECLARE_AND_GET_RE_DEBUG_FLAGS;
 
     PERL_ARGS_ASSERT_DUMP_TRIE_INTERIM_LIST;
@@ -159,7 +159,7 @@ S_dump_trie_interim_list(pTHX_ const struct _reg_trie_data *trie,
     Perl_re_indentf( aTHX_  "%s",
             depth+1, "------:-----+-----------------\n" );
 
-    for( state=1 ; state < next_alloc ; state ++ ) {
+    for( state = 1; state < next_alloc; state++ ) {
         U16 charid;
 
         Perl_re_indentf( aTHX_  " %4" UVXf " :",
@@ -208,8 +208,8 @@ S_dump_trie_interim_table(pTHX_ const struct _reg_trie_data *trie,
 {
     U32 state;
     U16 charid;
-    SV *sv=sv_newmortal();
-    int colwidth= widecharmap ? 6 : 4;
+    SV *sv = sv_newmortal();
+    int colwidth = widecharmap ? 6 : 4;
     DECLARE_AND_GET_RE_DEBUG_FLAGS;
 
     PERL_ARGS_ASSERT_DUMP_TRIE_INTERIM_TABLE;
@@ -238,20 +238,20 @@ S_dump_trie_interim_table(pTHX_ const struct _reg_trie_data *trie,
     Perl_re_printf( aTHX_ "\n");
     Perl_re_indentf( aTHX_  "State+-", depth+1 );
 
-    for( charid=0 ; charid < trie->uniquecharcount ; charid++ ) {
+    for( charid = 0; charid < trie->uniquecharcount; charid++ ) {
         Perl_re_printf( aTHX_  "%.*s", colwidth,"--------");
     }
 
     Perl_re_printf( aTHX_  "\n" );
 
-    for( state=1 ; state < next_alloc ; state += trie->uniquecharcount ) {
+    for( state = 1; state < next_alloc; state += trie->uniquecharcount ) {
 
         Perl_re_indentf( aTHX_  "%4" UVXf " : ",
             depth+1,
             (UV)TRIE_NODENUM( state ) );
 
         for( charid = 0 ; charid < trie->uniquecharcount ; charid++ ) {
-            UV v=(UV)SAFE_TRIE_NODENUM( trie->trans[ state + charid ].next );
+            UV v = (UV)SAFE_TRIE_NODENUM( trie->trans[ state + charid ].next );
             if (v)
                 Perl_re_printf( aTHX_  "%*" UVXf, colwidth, v );
             else
@@ -304,7 +304,7 @@ will be in parenthesis.
       |
       +-s->(3)-+-h->(4)-+-e->[5]
 
-      Accept Word Mapping: 3=>1 (he),5=>2 (she), 7=>3 (his), 9=>4 (hers)
+      Accept Word Mapping: 3 => 1 (he), 5 => 2 (she), 7 => 3 (his), 9 => 4 (hers)
 
 This shows that when matching against the string 'hers' we will begin at state 1
 read 'h' and move to state 2, read 'e' and move to state 3 which is accepting,
@@ -347,7 +347,7 @@ The regexp /(ac|ad|ab)+/ will produce the following debug output:
   17:   NOTHING(18)
   18: END(0)
 
-This would be optimizable with startbranch=5, first=5, last=16, tail=16
+This would be optimizable with startbranch = 5, first = 5, last = 16, tail = 16
 and should turn into:
 
    1: CURLYM[1] {1,32767}(18)
@@ -370,7 +370,7 @@ Cases where tail != last would be like /(?foo|bar)baz/:
    8: EXACT <baz>(10)
   10: END(0)
 
-which would be optimizable with startbranch=1, first=1, last=7, tail=8
+which would be optimizable with startbranch = 1, first = 1, last = 7, tail = 8
 and would end up looking like:
 
     1: TRIE(8)
@@ -381,7 +381,7 @@ and would end up looking like:
    8: EXACT <baz>(10)
   10: END(0)
 
-    d = uvchr_to_utf8_flags(d, uv, 0);
+    d = uv_to_utf8(d, uv);
 
 is the recommended Unicode-aware way of saying
 
@@ -393,7 +393,7 @@ is the recommended Unicode-aware way of saying
         if (UTF) {                                                         \
             SV *zlopp = newSV(UTF8_MAXBYTES);                              \
             unsigned char *flrbbbbb = (unsigned char *) SvPVX(zlopp);      \
-            unsigned char *const kapow = uvchr_to_utf8(flrbbbbb, val);     \
+            unsigned char *const kapow = uv_to_utf8(flrbbbbb, val);     \
             *kapow = '\0';                                                 \
             SvCUR_set(zlopp, kapow - flrbbbbb);                            \
             SvPOK_on(zlopp);                                               \
@@ -450,7 +450,7 @@ is the recommended Unicode-aware way of saying
 } STMT_END
 
 #define TRIE_HANDLE_WORD(state) STMT_START {                    \
-    U16 dupe= trie->states[ state ].wordnum;                    \
+    U16 dupe = trie->states[ state ].wordnum;                    \
     regnode * const noper_next = regnext( noper );              \
                                                                 \
     DEBUG_r({                                                   \
@@ -469,14 +469,30 @@ is the recommended Unicode-aware way of saying
     trie->wordinfo[curword].accept = state;                     \
                                                                 \
     if ( noper_next < tail ) {                                  \
-        if (!trie->jump)                                        \
+        if (!trie->jump) {                                      \
             trie->jump = (U16 *) PerlMemShared_calloc( word_count + 1, \
                                                  sizeof(U16) ); \
+            trie->j_before_paren = (U16 *) PerlMemShared_calloc( word_count + 1, \
+                                                 sizeof(U16) ); \
+            trie->j_after_paren = (U16 *) PerlMemShared_calloc( word_count + 1, \
+                                                 sizeof(U16) ); \
+        }                                                       \
         trie->jump[curword] = (U16)(noper_next - convert);      \
+        U16 set_before_paren;                                   \
+        U16 set_after_paren;                                    \
+        if (OP(cur) == BRANCH) {                                \
+            set_before_paren = ARG1a(cur);                       \
+            set_after_paren = ARG1b(cur);                        \
+        } else {                                                \
+            set_before_paren = ARG2a(cur);                     \
+            set_after_paren = ARG2b(cur);                      \
+        }                                                       \
+        trie->j_before_paren[curword] = set_before_paren;       \
+        trie->j_after_paren[curword] = set_after_paren;         \
         if (!jumper)                                            \
             jumper = noper_next;                                \
         if (!nextbranch)                                        \
-            nextbranch= regnext(cur);                           \
+            nextbranch = regnext(cur);                           \
     }                                                           \
                                                                 \
     if ( dupe ) {                                               \
@@ -498,24 +514,36 @@ is the recommended Unicode-aware way of saying
          && state == trie->trans[ base - ucharcount + charid ].check    \
          && trie->trans[ base - ucharcount + charid ].next )            \
            ? trie->trans[ base - ucharcount + charid ].next             \
-           : ( state==1 ? special : 0 )                                 \
+           : ( state == 1 ? special : 0 )                                 \
       )
 
-#define TRIE_BITMAP_SET_FOLDED(trie, uvc, folder)           \
-STMT_START {                                                \
-    TRIE_BITMAP_SET(trie, uvc);                             \
-    /* store the folded codepoint */                        \
-    if ( folder )                                           \
-        TRIE_BITMAP_SET(trie, folder[(U8) uvc ]);           \
-                                                            \
-    if ( !UTF ) {                                           \
-        /* store first byte of utf8 representation of */    \
-        /* variant codepoints */                            \
-        if (! UVCHR_IS_INVARIANT(uvc)) {                    \
-            TRIE_BITMAP_SET(trie, UTF8_TWO_BYTE_HI(uvc));   \
-        }                                                   \
-    }                                                       \
-} STMT_END
+/* Helper function for make_trie(): set a trie bit for both the character
+ * and its folded variant, and for the first byte of a variant codepoint,
+ * if any */
+
+STATIC void
+S_trie_bitmap_set_folded(pTHX_ RExC_state_t *pRExC_state,
+    reg_trie_data *trie, U8 ch, const U8 * folder)
+{
+    TRIE_BITMAP_SET(trie, ch);
+    /* store the folded codepoint */
+    if ( folder )
+        TRIE_BITMAP_SET(trie, folder[ch]);
+
+    if ( !UTF ) {
+        /* store first byte of utf8 representation of */
+        /* variant codepoints */
+        if (! UVCHR_IS_INVARIANT(ch)) {
+            U8 hi = UTF8_TWO_BYTE_HI(ch);
+            /* Note that hi will be either 0xc2 or 0xc3 (apart from EBCDIC
+             * systems), and TRIE_BITMAP_SET() will do >>3 to get the byte
+             * offset within the bit table, which is constant, and
+             * Coverity complained about this (CID 488118). */
+            TRIE_BITMAP_SET(trie, hi);
+        }
+    }
+}
+
 
 I32
 Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
@@ -533,6 +561,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
     U32 next_alloc = 0;
     regnode *jumper = NULL;
     regnode *nextbranch = NULL;
+    regnode *lastbranch = NULL;
     regnode *convert = NULL;
     U32 *prev_states; /* temp array mapping each state to previous one */
     /* we just use folder as a flag in utf8 */
@@ -549,7 +578,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
      */
 #else
     const U32 data_slot = reg_add_data( pRExC_state, STR_WITH_LEN("tu"));
-    STRLEN trie_charcount=0;
+    STRLEN trie_charcount = 0;
 #endif
     SV *re_trie_maxbuff;
     DECLARE_AND_GET_RE_DEBUG_FLAGS;
@@ -569,6 +598,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
         default: Perl_croak( aTHX_ "panic! In trie construction, unknown node type %u %s", (unsigned) flags, REGNODE_NAME(flags) );
     }
 
+    /* create the trie struct, all zeroed */
     trie = (reg_trie_data *) PerlMemShared_calloc( 1, sizeof(reg_trie_data) );
     trie->refcount = 1;
     trie->startstate = 1;
@@ -591,7 +621,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
     }
     DEBUG_TRIE_COMPILE_r({
         Perl_re_indentf( aTHX_
-          "make_trie start==%d, first==%d, last==%d, tail==%d depth=%d\n",
+          "make_trie start == %d, first == %d, last == %d, tail == %d depth = %d\n",
           depth+1,
           REG_NODE_NUM(startbranch), REG_NODE_NUM(first),
           REG_NODE_NUM(last), REG_NODE_NUM(tail), (int)depth);
@@ -640,6 +670,15 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
         bool set_bit = trie->bitmap ? 1 : 0; /*store the first char in the
                                                bitmap?*/
 
+        /* wordlen is needed for the TRIE_READ_CHAR() macro, but we don't use its
+           value in this scope, we only modify it.  clang 17 warns about this.
+           The later definitions of wordlen in this function do have their values
+           used.
+        */
+        PERL_UNUSED_VAR(wordlen);
+
+        lastbranch = cur;
+
         if (OP(noper) == NOTHING) {
             /* skip past a NOTHING at the start of an alternation
              * eg, /(?:)a|(?:b)/ should be the same as /a|b/
@@ -651,7 +690,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
 
             regnode *noper_next= regnext(noper);
             if (noper_next < tail)
-                noper= noper_next;
+                noper = noper_next;
         }
 
         if (    noper < tail
@@ -660,15 +699,15 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                 || (flags == EXACTFU && (   OP(noper) == EXACTFU_REQ8
                                          || OP(noper) == EXACTFUP))))
         {
-            uc= (U8*)STRING(noper);
-            e= uc + STR_LEN(noper);
+            uc = (U8*)STRING(noper);
+            e = uc + STR_LEN(noper);
         } else {
             trie->minlen= 0;
             continue;
         }
 
 
-        if ( set_bit ) { /* bitmap only alloced when !(UTF&&Folding) */
+        if ( set_bit ) { /* bitmap only alloced when !(UTF && Folding) */
             TRIE_BITMAP_SET(trie,*uc); /* store the raw first byte
                                           regardless of encoding */
             if (OP( noper ) == EXACTFUP) {
@@ -745,7 +784,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
              * branch */
             if ( uvc < 256 ) {
                 if ( folder ) {
-                    U8 folded= folder[ (U8) uvc ];
+                    U8 folded = folder[ (U8) uvc ];
                     if ( !trie->charmap[ folded ] ) {
                         trie->charmap[ folded ]=( ++trie->uniquecharcount );
                         TRIE_STORE_REVCHAR( folded );
@@ -758,7 +797,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                 if ( set_bit ) {
                     /* store the codepoint in the bitmap, and its folded
                      * equivalent. */
-                    TRIE_BITMAP_SET_FOLDED(trie, uvc, folder);
+                    S_trie_bitmap_set_folded(aTHX_ pRExC_state, trie, uvc, folder);
                     set_bit = 0; /* We've done our bit :-) */
                 }
             } else {
@@ -797,6 +836,13 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
             trie->maxlen = maxchars;
         }
     } /* end first pass */
+    trie->before_paren = OP(first) == BRANCH
+                 ? ARG1a(first)
+                 : ARG2a(first); /* BRANCHJ */
+
+    trie->after_paren = OP(lastbranch) == BRANCH
+                 ? ARG1b(lastbranch)
+                 : ARG2b(lastbranch); /* BRANCHJ */
     DEBUG_TRIE_COMPILE_r(
         Perl_re_indentf( aTHX_
                 "TRIE(%s): W:%d C:%d Uq:%d Min:%d Max:%d\n",
@@ -867,7 +913,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
             if (OP(noper) == NOTHING) {
                 regnode *noper_next= regnext(noper);
                 if (noper_next < tail)
-                    noper= noper_next;
+                    noper = noper_next;
                 /* we will undo this assignment if noper does not
                  * point at a trieable type in the else clause of
                  * the following statement. */
@@ -896,7 +942,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                         if ( !svpp ) {
                             charid = 0;
                         } else {
-                            charid=(U16)SvIV( *svpp );
+                            charid = (U16)SvIV( *svpp );
                         }
                     }
                     /* charid is now 0 if we dont know the char read, or
@@ -937,7 +983,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                  * on a trieable type. So we need to reset noper back to point at the first regop
                  * in the branch before we call TRIE_HANDLE_WORD()
                 */
-                noper= REGNODE_AFTER(cur);
+                noper = REGNODE_AFTER(cur);
             }
             TRIE_HANDLE_WORD(state);
 
@@ -964,8 +1010,8 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
             U32 zp = 0;
 
 
-            for( state=1 ; state < next_alloc ; state ++ ) {
-                U32 base=0;
+            for( state = 1; state < next_alloc; state ++ ) {
+                U32 base = 0;
 
                 /*
                 DEBUG_TRIE_COMPILE_MORE_r(
@@ -974,16 +1020,16 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                 */
 
                 if (trie->states[state].trans.list) {
-                    U16 minid=TRIE_LIST_ITEM( state, 1).forid;
-                    U16 maxid=minid;
+                    U16 minid = TRIE_LIST_ITEM( state, 1).forid;
+                    U16 maxid = minid;
                     U16 idx;
 
                     for( idx = 2 ; idx <= TRIE_LIST_USED( state ) ; idx++ ) {
                         const U16 forid = TRIE_LIST_ITEM( state, idx).forid;
                         if ( forid < minid ) {
-                            minid=forid;
+                            minid = forid;
                         } else if ( forid > maxid ) {
-                            maxid=forid;
+                            maxid = forid;
                         }
                     }
                     if ( transcount < tp + maxid - minid + 1) {
@@ -1017,7 +1063,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                             zp = tp;
                         }
                     } else {
-                        for ( idx=1; idx <= TRIE_LIST_USED( state ) ; idx++ ) {
+                        for ( idx = 1; idx <= TRIE_LIST_USED( state ) ; idx++ ) {
                             const U32 tid = base
                                            - trie->uniquecharcount
                                            + TRIE_LIST_ITEM( state, idx ).forid;
@@ -1034,7 +1080,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                     Perl_re_printf( aTHX_  " base: %d\n",base);
                 );
                 */
-                trie->states[ state ].trans.base=base;
+                trie->states[ state ].trans.base = base;
             }
             trie->lasttrans = tp + 1;
         }
@@ -1101,7 +1147,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
             if (OP(noper) == NOTHING) {
                 regnode *noper_next= regnext(noper);
                 if (noper_next < tail)
-                    noper= noper_next;
+                    noper = noper_next;
                 /* we will undo this assignment if noper does not
                  * point at a trieable type in the else clause of
                  * the following statement. */
@@ -1150,7 +1196,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                  * on a trieable type. So we need to reset noper back to point at the first regop
                  * in the branch before we call TRIE_HANDLE_WORD().
                 */
-                noper= REGNODE_AFTER(cur);
+                noper = REGNODE_AFTER(cur);
             }
             accept_state = TRIE_NODENUM( state );
             TRIE_HANDLE_WORD(accept_state);
@@ -1226,7 +1272,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
         */
         const U32 laststate = TRIE_NODENUM( next_alloc );
         U32 state, charid;
-        U32 pos = 0, zp=0;
+        U32 pos = 0, zp = 0;
         trie->statecount = laststate;
 
         for ( state = 1 ; state < laststate ; state++ ) {
@@ -1303,7 +1349,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
 
     {   /* Modify the program and insert the new TRIE node */
         U8 nodetype =(U8) flags;
-        char *str=NULL;
+        char *str = NULL;
 
 #ifdef DEBUGGING
         regnode *optimize = NULL;
@@ -1367,21 +1413,21 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                                 /* clear the bitmap */
                                 Zero(trie->bitmap, ANYOF_BITMAP_SIZE, char);
                                 DEBUG_OPTIMISE_r(
-                                    Perl_re_indentf( aTHX_  "New Start State=%" UVuf " Class: [",
+                                    Perl_re_indentf( aTHX_  "New Start State = %" UVuf " Class: [",
                                         depth+1,
                                         (UV)state));
                                 if (first_ofs >= 0) {
                                     SV ** const tmp = av_fetch_simple( revcharmap, first_ofs, 0);
                                     const U8 * const ch = (U8*)SvPV_nolen_const( *tmp );
 
-                                    TRIE_BITMAP_SET_FOLDED(trie,*ch, folder);
+                                    S_trie_bitmap_set_folded(aTHX_ pRExC_state, trie, *ch, folder);
                                     DEBUG_OPTIMISE_r(
                                         Perl_re_printf( aTHX_  "%s", (char*)ch)
                                     );
                                 }
                             }
                             /* store the current firstchar in the bitmap */
-                            TRIE_BITMAP_SET_FOLDED(trie,*ch, folder);
+                            S_trie_bitmap_set_folded(aTHX_ pRExC_state, trie, *ch, folder);
                             DEBUG_OPTIMISE_r(Perl_re_printf( aTHX_ "%s", ch));
                         }
                         first_ofs = ofs;
@@ -1395,8 +1441,8 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                     STRLEN len;
                     char *ch = SvPV( *tmp, len );
                     DEBUG_OPTIMISE_r({
-                        SV *sv=sv_newmortal();
-                        Perl_re_indentf( aTHX_  "Prefix State: %" UVuf " Ofs:%" UVuf " Char='%s'\n",
+                        SV *sv = sv_newmortal();
+                        Perl_re_indentf( aTHX_  "Prefix State: %" UVuf " Ofs: %" UVuf " Char: '%s'\n",
                             depth+1,
                             (UV)state, (UV)first_ofs,
                             pv_pretty(sv, SvPV_nolen_const(*tmp), SvCUR(*tmp), 6,
@@ -1406,9 +1452,9 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                             )
                         );
                     });
-                    if ( state==1 ) {
+                    if ( state == 1 ) {
                         OP( convert ) = nodetype;
-                        str=STRING(convert);
+                        str = STRING(convert);
                         setSTR_LEN(convert, 0);
                     }
                     assert( ( STR_LEN(convert) + len ) < 256 );
@@ -1417,7 +1463,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                         *str++ = *ch++;
                 } else {
 #ifdef DEBUGGING
-                    if (state>1)
+                    if (state > 1)
                         DEBUG_OPTIMISE_r(Perl_re_printf( aTHX_ "]\n"));
 #endif
                     break;
@@ -1466,7 +1512,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
             jumper = last;
         if ( trie->maxlen ) {
             NEXT_OFF( convert ) = (U16)(tail - convert);
-            ARG_SET( convert, data_slot );
+            ARG1u_SET( convert, data_slot );
             /* Store the offset to the first unabsorbed branch in
                jump[0], which is otherwise unused by the jump logic.
                We use this when dumping a trie and during optimisation. */
@@ -1491,7 +1537,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
                 OP( convert ) = TRIE;
 
             /* store the type in the flags */
-            convert->flags = nodetype;
+            FLAGS(convert) = nodetype;
             DEBUG_r({
             optimize = convert
                       + NODE_STEP_REGNODE
@@ -1526,7 +1572,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
         U32 state;
         U16 prev;
 
-        for (word=1; word <= trie->wordcount; word++) {
+        for (word = 1; word <= trie->wordcount; word++) {
             prev = 0;
             if (trie->wordinfo[word].prev)
                 continue;
@@ -1557,7 +1603,7 @@ Perl_make_trie(pTHX_ RExC_state_t *pRExC_state, regnode *startbranch,
 #endif
     return trie->jump
            ? MADE_JUMP_TRIE
-           : trie->startstate>1
+           : trie->startstate > 1
              ? MADE_EXACT_TRIE
              : MADE_TRIE;
 }
@@ -1588,8 +1634,8 @@ Perl_construct_ahocorasick_from_trie(pTHX_ RExC_state_t *pRExC_state, regnode *s
    'cdgu'.
  */
  /* add a fail transition */
-    const U32 trie_offset = ARG(source);
-    reg_trie_data *trie=(reg_trie_data *)RExC_rxi->data->data[trie_offset];
+    const U32 trie_offset = ARG1u(source);
+    reg_trie_data *trie = (reg_trie_data *)RExC_rxi->data->data[trie_offset];
     U32 *q;
     const U32 ucharcount = trie->uniquecharcount;
     const U32 numstates = trie->statecount;
@@ -1623,11 +1669,11 @@ Perl_construct_ahocorasick_from_trie(pTHX_ RExC_state_t *pRExC_state, regnode *s
     }
     OP(stclass)+=2; /* convert the TRIE type to its AHO-CORASICK equivalent */
 
-    ARG_SET( stclass, data_slot );
+    ARG1u_SET( stclass, data_slot );
     aho = (reg_ac_data *) PerlMemShared_calloc( 1, sizeof(reg_ac_data) );
     RExC_rxi->data->data[ data_slot ] = (void*)aho;
-    aho->trie=trie_offset;
-    aho->states=(reg_trie_state *)PerlMemShared_malloc( numstates * sizeof(reg_trie_state) );
+    aho->trie = trie_offset;
+    aho->states = (reg_trie_state *)PerlMemShared_malloc( numstates * sizeof(reg_trie_state) );
     Copy( trie->states, aho->states, numstates, reg_trie_state );
     Newx( q, numstates, U32);
     aho->fail = (U32 *) PerlMemShared_calloc( numstates, sizeof(U32) );
@@ -1680,7 +1726,7 @@ Perl_construct_ahocorasick_from_trie(pTHX_ RExC_state_t *pRExC_state, regnode *s
         Perl_re_indentf( aTHX_  "Stclass Failtable (%" UVuf " states): 0",
                       depth, (UV)numstates
         );
-        for( q_read=1; q_read<numstates; q_read++ ) {
+        for( q_read = 1; q_read < numstates; q_read++ ) {
             Perl_re_printf( aTHX_  ", %" UVuf, (UV)fail[q_read]);
         }
         Perl_re_printf( aTHX_  "\n");
