@@ -26,6 +26,7 @@ use warnings;
 my %feature = (
     say                     => 'say',
     state                   => 'state',
+    switch                  => 'switch',
     bitwise                 => 'bitwise',
     evalbytes               => 'evalbytes',
     current_sub             => '__SUB__',
@@ -48,6 +49,7 @@ my %feature = (
     apostrophe_as_package_separator => 'apos_as_name_sep',
     any                     => 'any',
     all                     => 'all',
+    smartmatch              => 'smartmatch',
 );
 
 # NOTE: If a feature is ever enabled in a non-contiguous range of Perl
@@ -57,20 +59,22 @@ my %feature = (
 # 5.odd implies the next 5.even, but an explicit 5.even can override it.
 
 # features bundles
-use constant V5_9_5 => sort qw{say state indirect multidimensional bareword_filehandles apostrophe_as_package_separator};
+use constant V5_9_5 => sort qw{say state switch indirect multidimensional bareword_filehandles apostrophe_as_package_separator smartmatch};
 use constant V5_11  => sort ( +V5_9_5, qw{unicode_strings} );
 use constant V5_15  => sort ( +V5_11, qw{unicode_eval evalbytes current_sub fc} );
 use constant V5_23  => sort ( +V5_15, qw{postderef_qq} );
 use constant V5_27  => sort ( +V5_23, qw{bitwise} );
 
-use constant V5_35  => sort grep {; $_ ne 'indirect'
+use constant V5_35  => sort grep {; $_ ne 'switch'
+                                 && $_ ne 'indirect'
                                  && $_ ne 'multidimensional' } +V5_27, qw{isa signatures};
 
 use constant V5_37  => sort grep {; $_ ne 'bareword_filehandles' } +V5_35, qw{module_true};
 
 use constant V5_39  => sort ( +V5_37, qw{try} );
 use constant V5_41  => sort
-  grep {; $_ ne 'apostrophe_as_package_separator' }
+  grep {; $_ ne 'apostrophe_as_package_separator'
+       && $_ ne 'smartmatch' }
   ( +V5_39 );
 
 #
@@ -79,7 +83,7 @@ use constant V5_41  => sort
 my %feature_bundle = (
     all     => [ sort keys %feature ],
     default => [ qw{indirect multidimensional bareword_filehandles
-                    apostrophe_as_package_separator} ],
+                    apostrophe_as_package_separator smartmatch} ],
     # using 5.9.5 features bundle
     "5.9.5" => [ +V5_9_5 ],
     "5.10"  => [ +V5_9_5 ],
@@ -110,7 +114,7 @@ my %feature_bundle = (
 );
 
 my @noops = qw( postderef lexical_subs );
-my @removed = qw( array_base switch );
+my @removed = qw( array_base );
 
 
 ###########################################################################
@@ -609,7 +613,7 @@ read_only_bottom_close_and_rename($h);
 
 __END__
 package feature;
-our $VERSION = '1.93';
+our $VERSION = '1.94';
 
 FEATURES
 
@@ -699,12 +703,40 @@ See L<perlsub/"Persistent Private Variables"> for details.
 
 This feature is available starting with Perl 5.10.
 
+=head2 The 'smartmatch' feature
+
+C<use feature 'smartmatch'> tells the compiler to enable the
+smartmatch operator C<~~>.  It is enabled by default, but can be
+turned off to disallow the C<~~> operator.
+
+This feature is disabled by default in the 5.42 feature bundle
+onwards:
+
+  $x ~~ $y; # fine
+  use v5.42;
+  $x ~~ $y; # error
+
+This has no effect on the implicit smartmatches done by C<when>.
+
+See L<perlop/"Smartmatch Operator"> for details.
+
 =head2 The 'switch' feature
 
-C<use feature 'switch'> told the compiler to enable the Raku
+B<WARNING>: This feature is still experimental and the implementation may
+change or be removed in future versions of Perl.  For this reason, Perl will
+warn when you use the feature, unless you have explicitly disabled the warning:
+
+    no warnings "experimental::smartmatch";
+
+C<use feature 'switch'> tells the compiler to enable the Raku
 given/when construct.
 
-This feature was removed in Perl 5.42.
+See L<perlsyn/"Switch Statements"> for details.
+
+This feature is available starting with Perl 5.10.
+It is deprecated starting with Perl 5.38, and using
+C<given>, C<when> or smartmatch will throw a warning.
+It will be removed in Perl 5.42.
 
 =head2 The 'unicode_strings' feature
 
